
import java.io.*;
import org.xml.sax.SAXException;
import org.w3c.dom.*;
import javax.xml.parsers.*;

public class DOMRSS2HTML2 {

  public static void main(String[] args) {
    try {
      convert(args[0], args[1]);
    }
    catch (SAXException e) {
      System.err.println("ERROR: " + e);
    }
    catch (ParserConfigurationException e) {
      System.err.println("ERROR: " + e);
    }
    catch (IOException e) {
      System.err.println("ERROR: " + e);
    }
  }

  public static void convert(String infile, String outfile)
    throws ParserConfigurationException, SAXException, IOException {
    Document doc = buildDocument(infile);
    FileWriter out = new FileWriter(outfile);

    Element root = doc.getDocumentElement();
    Element channel = getFirstChild("channel", root);
    String title = getData(getFirstChild("title", channel));
    String link = getData(getFirstChild("title", channel));
    String descr = getData(getFirstChild("description", channel));

    out.write("<title>" + title + "</title>\n");
    out.write("<h1><a href\"" + link + "\">" + title + "</a></h1>\n");
    if (descr != null)
      out.write("<p>" + descr + "</p>\n");

    // loop over item elements
    out.write("<ul>\n");
    NodeList children = channel.getChildNodes();
    for (int ix = 0; ix < children.getLength(); ix++) {
      Node node = children.item(ix);
      if (node.getNodeType() == Node.ELEMENT_NODE &&
	  node.getNodeName().equals("item"))
	convertItem(out, (Element) node);
    }
    out.write("</ul>\n");

    out.close();
  }

  public static void convertItem(Writer out, Element item)
    throws IOException {
    String title = getData(getFirstChild("title", item));
    String link = getData(getFirstChild("title", item));
    String descr = getData(getFirstChild("description", item));
    
    out.write("  <li><a href=\"" + link + "\">" + title + "</a>");
    if (descr != null) out.write(" " + descr);
    out.write("</li>\n");
  }

  // --- DOM access utilities

  public static Element getFirstChild(String name, Node parent) {
    NodeList children = parent.getChildNodes();
    for (int ix = 0; ix < children.getLength(); ix++) {
      Node node = children.item(ix);
      if (node.getNodeType() == Node.ELEMENT_NODE &&
	  node.getNodeName().equals(name))
	return (Element) node;
      if (node.getNodeType() == Node.ENTITY_REFERENCE_NODE) {
	node = getFirstChild(name, node);
	if (node != null)
	  return (Element) node;
      }
    }

    return null;
  }

  public static String getData(Element parent) {
    if (parent == null)
      return null;
    
    StringBuffer content = new StringBuffer();
    NodeList children = parent.getChildNodes();
    for (int ix = 0; ix < children.getLength(); ix++) {
      Node node = children.item(ix);
      if (node.getNodeType() == Node.TEXT_NODE ||
	  node.getNodeType() == Node.CDATA_SECTION_NODE)
	content.append(node.getNodeValue());
    }
    return content.toString();
  }

  // --- DOM tree building

  private static Document buildDocument(String infile)
    throws ParserConfigurationException, SAXException, IOException {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    return builder.parse(new File(infile));
  }
  
}
