
// $Id: SAXRSS2HTML.java,v 1.1 2001/03/18 17:05:47 larsga Exp $

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Stack;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class SAXRSS2HTML {
  
  public static void main(String[] args) {
    try {
      convert(args[0], args[1]);
    }
    catch (ParserConfigurationException e) {
      System.err.println("ERROR: " + e);
    }
    catch (SAXException e) {
      System.err.println("ERROR: " + e);
    }
    catch (java.net.MalformedURLException e) {
      System.err.println("ERROR: " + e);
    }
    catch (IOException e) {
      System.err.println("ERROR: " + e);
    }
  }

  public static void convert(String infile, String outfile)
  throws java.net.MalformedURLException, IOException, SAXException,
	 ParserConfigurationException {
    XMLReader parser = getXMLReader();
    parser.setContentHandler(new RSSHandler(outfile));
    parser.parse(new File(infile).toURL().toString());
  }
  
  private static XMLReader getXMLReader()
    throws SAXException, ParserConfigurationException {
    SAXParserFactory factory = SAXParserFactory.newInstance();
    SAXParser parser = factory.newSAXParser();
    return parser.getXMLReader();
  }  

  // --- Internal classes

  static class RSSHandler extends DefaultHandler {
    private Writer  out;

    private Stack elements;    
    private boolean keepContent;
    private StringBuffer content;

    private boolean firstItem;
    private String title;
    
    public RSSHandler(String outfile) throws IOException {
      out = new FileWriter(outfile);
      elements = new Stack();
      firstItem = true;
    }

    public void startElement(String nsuri, String lname, String qname,
			     Attributes attrs) {
      keepContent = lname.equals("title") ||
	            lname.equals("link") ||
	            lname.equals("description");
      elements.push(lname);      
      
      if (keepContent)
	content = new StringBuffer();
    }

    public void endElement(String nsuri, String lname, String qname)
    throws SAXException {
      try {
	keepContent = false;
	elements.pop();

	if (elements.isEmpty())
	  return;

	String parent = (String) elements.peek();

	// CHANNEL
	if (parent.equals("channel")) {
	  if (lname.equals("title"))
	    title = content.toString();

	  else if (lname.equals("link")) {
	    out.write("<title>" + title + "</title>\n");
	    out.write("<h1><a href=\"" + content + "\">" + title +
		      "</a></h1>\n");
	    
	  } else if (lname.equals("description")) 
	    out.write("<p>" + content + "</p>\n");

        // ITEM
	} else if (parent.equals("item")) {
	  if (firstItem) {
	    out.write("\n<ul>");
	    firstItem = false;
	  }
	  
	  if (lname.equals("title"))
	    title = content.toString();

	  else if (lname.equals("link")) 
	    out.write("\n<li><a href=\"" + content + "\">" + title +
		      "</a>");
	    
	  else if (lname.equals("description")) 
	    out.write(content.toString());

	  
	} else if (lname.equals("channel"))
	  out.write("\n</ul>\n");
      }
      catch (IOException e) {
	throw new SAXException(e);
      }
    }
    
    public void characters(char[] chars, int start, int length) {
      if (keepContent)
	content.append(chars, start, length);
    }
    
    public void endDocument() throws SAXException {
      try {
	out.close();
      }
      catch (IOException e) {
	throw new SAXException(e);
      }
    }
  }
  
}

