
// $Id: SAXRSS2HTML2.java,v 1.1 2001/03/18 22:43:57 larsga Exp $

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Stack;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class SAXRSS2HTML2 {
  
  public static void main(String[] args) {
    try {
      convert(args[0], args[1]);
    }
    catch (ParserConfigurationException e) {
      System.err.println("ERROR: " + e);
    }
    catch (SAXException e) {
      System.err.println("ERROR: " + e);
    }
    catch (java.net.MalformedURLException e) {
      System.err.println("ERROR: " + e);
    }
    catch (IOException e) {
      System.err.println("ERROR: " + e);
    }
  }

  public static void convert(String infile, String outfile)
  throws java.net.MalformedURLException, IOException, SAXException,
	 ParserConfigurationException {
    XMLReader parser = getXMLReader();
    parser.setContentHandler(new RSSHandler(outfile));
    parser.parse(new File(infile).toURL().toString());
  }
  
  private static XMLReader getXMLReader()
    throws SAXException, ParserConfigurationException {
    SAXParserFactory factory = SAXParserFactory.newInstance();
    SAXParser parser = factory.newSAXParser();
    return parser.getXMLReader();
  }  

  // --- Internal classes

  static class RSSHandler extends SAXTracker {
    private Writer  out;

    private boolean firstItem;
    private String title;
    
    public RSSHandler(String outfile) throws IOException {
      out = new FileWriter(outfile);
      firstItem = true;
      keepContentsOf("title");
      keepContentsOf("link");
      keepContentsOf("description");
    }

    public void endElement(String nsuri, String lname, String qname) 
    throws SAXException {
      super.endElement(nsuri, lname, qname);
      
      try {
	// CHANNEL
	if (isParent("channel")) {
	  if (lname.equals("title"))
	    title = content.toString();

	  else if (lname.equals("link")) {
	    out.write("<title>" + title + "</title>\n");
	    out.write("<h1><a href=\"" + content + "\">" + title +
		      "</a></h1>\n");
	    
	  } else if (lname.equals("description")) 
	    out.write("<p>" + content + "</p>\n");

        // ITEM
	} else if (isParent("item")) {
	  if (firstItem) {
	    out.write("\n<ul>");
	    firstItem = false;
	  }
	  
	  if (lname.equals("title"))
	    title = content.toString();

	  else if (lname.equals("link")) 
	    out.write("\n<li><a href=\"" + content + "\">" + title +
		      "</a>");
	    
	  else if (lname.equals("description")) 
	    out.write(content.toString());

	  
	} else if (lname.equals("channel"))
	  out.write("\n</ul>\n");
      }
      catch (IOException e) {
	throw new SAXException(e);
      }
    }
    
    public void endDocument() throws SAXException {
      try {
	out.close();
      }
      catch (IOException e) {
	throw new SAXException(e);
      }
    }
  }
  
}

