
import xbellib
    
def xbel2dom(xbel, doc):
    """This function takes an XBEL structure and a DOM Document Node
    and creates a DOM tree reflecting the XBEL object structure."""
    
    doc.appendChild(doc.createElement("xbel"))
    root = doc.documentElement

    if xbel.get_title():
        title = doc.createElement("title")
        title.appendChild(doc.createTextNode(xbel.get_title()))
        root.appendChild(title)

    if xbel.get_descr():
        descr = doc.createElement("descr")
        descr.appendChild(doc.createTextNode(xbel.get_descr()))
        root.appendChild(descr)

    for child in xbel.get_nodes():
        root.appendChild(create_node(child, doc))    

def create_node(node, doc):
    if isinstance(node, xbellib.Bookmark) or \
       isinstance(node, xbellib.Folder):
        if isinstance(node, xbellib.Bookmark):
            elem = doc.createElement("bookmark")
            elem.setAttribute("href", node.get_url())
        else:
            elem = doc.createElement("folder")

        if node.get_title():
            title = doc.createElement("title")
            title.appendChild(doc.createTextNode(node.get_title()))
            elem.appendChild(title)

        if node.get_descr():
            descr = doc.createElement("descr")
            descr.appendChild(doc.createTextNode(node.get_descr()))
            elem.appendChild(descr)
                    
        if isinstance(node, xbellib.Folder):
            for child in node.get_nodes():
                elem.appendChild(create_node(child, doc))

        return elem
    
    elif isinstance(node, xbellib.Separator):
        return doc.createElement("separator")
    
    elif isinstance(node, xbellib.Alias):
        return create_node(node.get_ref_node(), doc)

    else:
        raise ValueError("Node of unknown type encountered! %s" %
                         node)

def serialize_4DOM(xbel, filename):
    from xml.dom import ext, Document

    doc = Document.Document(None)
    xbel2dom(xbel, doc)
    
    outf = open(filename, "w")
    ext.Print(doc, outf)
    outf.close()
    
if __name__ == "__main__":
    import sys
    serialize_4DOM(xbellib.load_xbel(sys.argv[1]), sys.argv[2])
