import sys, string

from eventdom.eventdom import *
from domutils import *

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
  <LINK REL=stylesheet HREF="play.css" TYPE="text/css">
</HEAD>

<BODY>
<H1>%s</H1>
"""

bottom = \
"""
<HR>
<ADDRESS>
Converted to HTML from Jon Bosak's XML by play2html, using eventdom.
</ADDRESS>

</BODY>
</HTML>
"""

# --- Handler class

class PlayHandler(EventDOMHandler):

    def __init__(self, out = None):
        EventDOMHandler.__init__(self)
        self._out = out or sys.stdout

    # front matter

    def fm_para(self, node):
        "process: P"
        self._out.write("<p>%s</p>" % data(node))


    def play_title(self, node):
        "process: PLAY/TITLE"
        title = data(node)
        self._out.write(top % (title, title))


    def personae_title(self, node):
        "process: PERSONAE/TITLE"
        self._out.write("<h2>%s</h2>\n" % data(node))
        self._out.write("<ul>\n")

    def persona(self, node):
        "process: PERSONA"
        self._out.write("  <li>%s</li>\n" % data(node))


    def pgroup(self, node):
        "process: PGROUP"

        descr = data(find_first_child(node, ELEMENT_NODE, "GRPDESCR"))
        if descr[-1] == ".":
            end = -1
        else:
            end = len(descr)
        descr = string.upper(descr[0]) + descr[1 : end]
        self._out.write("  <li><b>%s:</b>\n<ul>" % descr)

        for person in filter(predicate(ELEMENT_NODE, "PERSONA"),
                             node.childNodes):
            self._out.write("  <li>%s</li>\n" % data(person))

        self._out.write("</ul></li>\n")


    def personae(self, node):
        "end: PERSONAE"
        self._out.write("</ul>\n")

    # structure

    def act_title(self, node):
        "process: ACT/TITLE"
        self._out.write("<h2>%s</h2>\n" % data(node))

    def scene_title(self, node):
        "process: SCENE/TITLE"
        self._out.write("<h3>%s</h3>\n" % data(node))

    # content
        
    def speaker(self, node):
        "process: SPEAKER"
        self._out.write("<p class=speaker>%s</p>" % data(node))

    def line(self, node):
        "process: LINE"
        self._out.write("<p class=line>%s</p>" % data(node))

    def speech_stagedir(self, node):
        "process: SPEECH/STAGEDIR"
        self._out.write("<p class=line>%s</p>" % data(node))

    def line_stagedir(self, node):
        "process: LINE/STAGEDIR"
        self._out.write("<i>%s</i>" % data(node))

    # end

    def play_end(self, node):
        "end: PLAY"
        self._out.write(bottom)
