import sys
from eventdom.eventdom import *
from domutils import *

# --- HTML templates

top = \
"""
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<H1>%s</H1>
"""

bottom = \
"""        
<HR>

<ADDRESS>
Converted by xbel2html.py, using eventdom.
</ADDRESS>

</BODY>
</HTML>
"""


# --- Handler class

class XBEL2HTML(EventDOMHandler):

    def __init__(self, out = None):
        EventDOMHandler.__init__(self)
        self._out = out or sys.stdout
        self._title = "XBEL bookmark collection"
        self._descr = None


    def title(self, node):
        ' process: xbel/title '
        self._title = data(node)

    def descr(self, node):
        ' process: xbel/descr '
        self._descr = data(node)


    def bookmark(self, node):
        ' process: bookmark '
        if self._title:
            self._write_top()
        
        title = data(find_first_child(node, ELEMENT_NODE, 'title'),
                     '<none>')
        link  = node.getAttribute('href')
        descr = data(find_first_child(node, ELEMENT_NODE, 'descr'),
                     '')

        self._out.write('  <li><a href="%s">%s</a> %s</li>\n' %
                        (link, title, descr))


    def folder_title(self, node):
        ' process: folder/title '
        if self._title:
            self._write_top()
            
        depth = self._find_depth(node)
        title = data(node)

        if depth == 0:
            self._out.write('<h1>%s</h1>' % title)
            self._out.write('<ul>\n')
            
        else:
            self._out.write('  <li><b>%s</b>\n' % title)
            self._out.write('  <ul>\n')


    def end_folder(self, node):
        ' end: folder '
        self._out.write('</ul>\n')

    def end_xbel(self, node):
        ' end: xbel '
        self._out.write(bottom)


    # internal methods

    def _write_top(self):
        self._out.write(top % (self._title, self._title))
        if self._descr:
            self._out.write("<p>%s</p>\n" % self._descr)
            
        self._title = None


    def _find_depth(self, node):
        node = node.parentNode.parentNode # must get out of title...
        depth = 0
        
        while node.nodeName != 'xbel':
            depth = depth + 1
            node = node.parentNode

        return depth


# --- Main program

def _test(file):
    parse_file(file, XBEL2HTML())

if __name__ == "__main__":
    _test(sys.argv[1])
