"""
How to implement external entity support for pyexpat.
"""

import sys, urllib, urlparse
from xml.parsers import expat

# --- The application

class ExternalEntityHandling:

    def __init__(self, parser):
        self._parsers = [parser]

        
    def external_entity_handler(self, context, base, sysid, pubid):
        new_parser = self._parsers[-1].ExternalEntityParserCreate(context)
        self._parsers.append(new_parser)
        
        source = urllib.urlopen(urlparse.urljoin(base, sysid))
        buf = source.read(16384)
        while buf != "":
            if new_parser.Parse(buf, 0) != 1:
                return 0
            
            buf = source.read(16384)

        del self._parsers[-1]
        return 1

    
# --- A test driver

def element_start(name, atts):
    print "Start:", name

def _test(sysid):
    p = expat.ParserCreate()
    p.SetBase(sysid) # necessary to tell expat the base URI
    app = ExternalEntityHandling(p)
    p.StartElementHandler = element_start
    p.ExternalEntityRefHandler = app.external_entity_handler

    error = 0
    inf = open(sysid)
    buf = inf.read(16384)
    while buf != "":
        if p.Parse(buf, 0) != 1:
            error = 1
            break
        buf = inf.read(16384)

    inf.close()

    if error or p.Parse("", 1) != 1:
        print "ERROR: %s in %s:%s:%s" % (expat.ErrorString(p.ErrorCode),
                                         sysid, p.ErrorLineNumber,
                                         p.ErrorColumnNumber)
    
if __name__ == "__main__":
    _test(sys.argv[1])

