"""
This application uses pyexpat to convert RSS documents to HTML.
"""

import sys, codecs
from xml.parsers import expat

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>

<H1><A HREF="%s">%s</A></H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted to HTML, by expat_rss2html.py.
</ADDRESS>

</BODY>
</HTML>
"""

# --- The converter

class RSS2HTML:

    def __init__(self, out = None):
        self._out = out or sys.stdout
        
        # tracking state
        self._data = ""
        self._first_item = 1

        self._title = None
        self._link = None
        self._descr = None
        
    def start_tag(self, name, attrs):
        self._data = ""

        if name == "item":
            self._descr = None # reset for this item
            if self._first_item:
                self._out.write("\n<ul>\n")

            self._first_item = 0
            
    def end_tag(self, name):
        if name == "title":
            self._title = self._data

        elif name == "link":
            self._link = self._data

        elif name == "descr":
            self._descr = self._data

        elif name == "channel":
            self._out.write(top % (self._title, self._link, self._title))

            if self._descr != None:
                self._out.write("<p>%s</p>" % self._descr)

        elif name == "item":
            self._out.write('  <li><a href="%s">%s</a> %s\n' %
                           (self._link,self._title,self._descr or ""))

        elif name == "rss":
            self._out.write("</ul>\n")
            self._out.write(bottom)
            
    def data_handler(self, data):
        self._data = self._data + data

# --- The driver

def convert(sysid, out):
    app = RSS2HTML(out)
    p = expat.ParserCreate()
    p.StartElementHandler = app.start_tag
    p.EndElementHandler = app.end_tag
    p.CharacterDataHandler = app.data_handler

    error = 0
    inf = open(sysid)
    buf = inf.read(16384)
    while buf != "":
        if p.Parse(buf, 0) != 1:
            error = 1
            break
        buf = inf.read(16384)

    inf.close()
    
    if error or p.Parse("", 1) != 1:
        print "ERROR: %s in %s:%s:%s" % (expat.ErrorString(p.ErrorCode),
                                         sysid, p.ErrorLineNumber,
                                         p.ErrorColumnNumber)

# --- The main program

sysid = sys.argv[1]
out   = codecs.open(sys.argv[2], "w", "iso-8859-1")

convert(sysid, out)

out.close()
