
import sys, string
import pysp

# --- Utilities

def normalize(str):
    return string.join(string.split(str), " ")

# --- The application object

class ToCHandler:

    def __init__(self, out = None):
        self._out = out or sys.stdout
        self._numbers = [0]
        self._elemstack = []
        self._contents = None


    def error(self, msg):
        sys.stderr.write(msg + "\n")

    def start_element(self, name, attrs):
        self._elemstack.append(name)
        if name in ("TITLE", "H1", "H2", "H3", "H4"):
            self._contents = ""
        
    def data(self, data):
        if "TITLE" in self._elemstack or \
           "H1" in self._elemstack or \
           "H2" in self._elemstack or \
           "H3" in self._elemstack or \
           "H4" in self._elemstack:
            self._contents = self._contents + data

    def end_element(self, name):
        del self._elemstack[-1]

        level = 0
        if name == "TITLE" and self._elemstack[-1] in \
           ("CHAPOPEN", "APPENDIX"):
            self._numbers = [self._numbers[0] + 1]
            self._out.write("\n%d. %s\n" % (self._numbers[0],
                                            normalize(self._contents)))

        elif name == "H1":
            level = 1
        elif name == "H2":
            level = 2
        elif name == "H3":
            level = 3

        if level:
            if len(self._numbers) == level:
                self._numbers = self._numbers[ : level] + [1]
            else:
                self._numbers = self._numbers[ : level] + \
                                [self._numbers[level] + 1]

            self._out.write(("  " * level))
            for number in self._numbers[ : level + 1]:
                self._out.write("%d." % number)
                
            self._out.write(" %s\n" % normalize(self._contents))


            
# --- Main program

if __name__ == "__main__":
    import os
    if os.environ.has_key("SGML_CATALOG_FILES"):
        for catalog in string.split(os.environ["SGML_CATALOG_FILES"],
                                    os.pathsep):
            pysp.add_catalog(catalog)
        
    parser = pysp.make_parser(sys.argv[1])
    parser.run(ToCHandler())

