
import sys, codecs
from pyxie import xTree, File2PYX, PYXDecoder

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<A HREF="%s"><H1>%s</H1></A>
"""

bottom = \
"""
</UL>

<HR>

<ADDRESS>
Converted to HTML using Pyxie.
</ADDRESS>

</BODY>
</HTML>
"""

# --- xTree application

def find_right(tree, elem_name):
    while tree.HasRight() and not tree.AtElement(elem_name):
        tree.Right()

        
def extract_fields(tree):
    find_right(tree, "title")
    title = PYXDecoder(tree.JoinData(""))
    
    find_right(tree, "link")
    link  = PYXDecoder(tree.JoinData(""))

    find_right(tree, "description")
    if tree.AtElement("description"):
        descr = PYXDecoder(tree.JoinData(""))
    else:
        descr = ""

    return (title, link, descr)    


def convert(tree, out = None):
    out = out or sys.stdout
    
    tree.Down() # set current node to first <rss> child
    find_right(tree, "channel")
    tree.Down() # set current node to first <channel> child    
    tree.PushPos() # remember this position

    (title, link, descr) = extract_fields(tree)
    out.write(top % (title, link, title))
    if descr:
        out.write("<p>%s</p>" % descr)
    out.write("<ul>\n")

    tree.PopPos() # we're now back, in case find_right took us too far
    find_right(tree, "item")
    while tree.AtElement("item"):
        tree.Down() # go to first child of <item>
        (title, link, descr) = extract_fields(tree)
        out.write('<li><a href="%s">%s</a> %s\n' %
                  (link, title, descr))
        tree.Up() # come back up
        tree.Right() # step one past this <item>
        find_right(tree, "item")

    out.write(bottom)

    
# --- Main program

pyxfile = File2PYX(sys.argv[1])

tree = xTree()
tree.PYX2xTree(pyxfile)

convert(tree, codecs.open(sys.argv[2], "w", "iso-8859-1"))
