
import sys, codecs
from pyxie import xTree, File2PYX, PYXDecoder

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<A HREF="%s"><H1>%s</H1></A>
"""

bottom = \
"""
</UL>

<HR>

<ADDRESS>
Converted to HTML using Pyxie.
</ADDRESS>

</BODY>
</HTML>
"""

# --- xTree application

class DispatchHandler:

    def __init__(self, tree, out):
        self._cur = tree
        self._out = out

    def handle_channel(self, before):
        if not before:
            return

        (title, link, descr) = extract_fields(self._cur)
        self._out.write(top % (title, link, title))
        if descr:
            self._out.write("<p>%s</p>" % descr)
        self._out.write("<ul>\n")

    def handle_item(self, before):
        if not before:
            return

        (title, link, descr) = extract_fields(self._cur)
        self._out.write('<li><a href="%s">%s</a> %s\n' % (link, title, descr))

        
def find_right(tree, elem_name):
    while tree.HasRight() and not tree.AtElement(elem_name):
        tree.Right()

def extract_fields(tree):
    tree.PushPos()
    tree.Down()
    
    find_right(tree, "title")
    title = PYXDecoder(tree.JoinData(""))
    
    find_right(tree, "link")
    link  = PYXDecoder(tree.JoinData(""))

    find_right(tree, "description")
    if tree.AtElement("description"):
        descr = PYXDecoder(tree.JoinData(""))
    else:
        descr = ""

    tree.PopPos()
    return (title, link, descr)


# --- Main program

pyxfile = File2PYX(sys.argv[1])

tree = xTree()
tree.PYX2xTree(pyxfile)

outf = codecs.open(sys.argv[2], "w", "iso-8859-1")
tree.Dispatch(DispatchHandler(tree, outf))
sys.stdout.write(bottom)
