# This filter inherits certain attributes from parent elements onto
# their children.

from xml.sax.saxutils import XMLFilterBase
from xml.sax.xmlreader import AttributesImpl

class MutableAttributesImpl(AttributesImpl):

    def __init__(self, attrs):
	localattrs = {}
	for (k, v) in attrs.items():
            localattrs[k] = v
        AttributesImpl.__init__(self, localattrs)

    def __setitem__(self, key, value):
        self._attrs[key] = value


class AttributeInheritanceFilter(XMLFilterBase):

    # initialization
    
    def __init__(self, parent, inherited):
        XMLFilterBase.__init__(self, parent)
        self._inherited = inherited # dict w/ inherited attrs as keys
        self._attstack = [{}]

    # events
    
    def startElement(self, name, attrs):
        inherit = self._attstack[-1].copy()
        for inherited in self._inherited.keys():
            if attrs.has_key(inherited):
                inherit[inherited] = attrs[inherited]

        attrs = MutableAttributesImpl(attrs)
        for (k, v) in self._attstack[-1].items():
            if not attrs.has_key(k):
                attrs[k] = v
            
        self._cont_handler.startElement(name, attrs)
        self._attstack.append(inherit)
    
    def endElement(self, name):
        del self._attstack[-1]
        self._cont_handler.endElement(name)

