import sys
from xml.sax import parse
from saxtracker import SAXTracker

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
  <LINK REL=stylesheet HREF="play.css" TYPE="text/css">
</HEAD>

<BODY>

<H1>%s</H1>
"""

bottom = \
"""
<HR>

<ADDRESS>
Converted from Jon Bosak's XML to HTML with SAX.
</ADDRESS>

</BODY>
</HTML>
"""

# --- The ContentHandler

cutoff = {"SPEECH" : 1, "SPEAKER" : 1, "STAGEDIR" : 1, "LINE" : 1}

class PlayToHtmlHandler(SAXTracker):

    def __init__(self, out = None):
        SAXTracker.__init__(self, ["TITLE", "P", "PERSONA", "GRPDESCR",
                                   "SPEAKER", "LINE", "STAGEDIR"])
        self._write = (out or sys.stdout).write

    def startElement(self, name, attrs):
        if cutoff.has_key(name):
            SAXTracker.startElement(self, name, attrs)
            return
        
        # front matter
        
        if name == "FM": 
            self._write("<h2>Front matter</h2>\n")

        elif name == "PERSONAE":
            self._write("<h2>Dramatis personae</h2>\n")
            self._write("<ul>")

        elif name == "PGROUP":
            self._write("<li>")

        # play

        elif name == "PROLOGUE":
            self._write("<h2>Prologue</h2>\n")
        
        elif name == "EPILOGUE":
            self._write("<h2>Epilogue</h2>\n")
        
        SAXTracker.startElement(self, name, attrs)
        
    def endElement(self, name):
        SAXTracker.endElement(self, name)

        if name == "LINE":
            self._write("<p class=line>%s</p>" % self._contents)

        elif name == "SPEECH":
            pass

        elif name == "SPEAKER" and \
             self._elemstack[-2] != "EPILOGUE" and \
             self._elemstack[-2] != "PROLOGUE":
                self._write("<p class=speaker>%s:</p>\n" %
                            self._contents)
            
        elif name == "STAGEDIR":
            self._write("<p class=stagedir>%s</p>" % self._contents)
            
        elif name == "TITLE":
            if self._elemstack[-1] == "SCENE":
                self._write("<h3>%s</h3>" % self._contents)
                
            elif self._elemstack[-1] == "ACT" or \
                 self._elemstack[-1] == "INDUCT":
                self._write("<h2>%s</h2>" % self._contents)

            elif self._elemstack[-1] == "PLAY":
                self._write(top % (self._contents, self._contents))

        elif name == "P":
            self._write("<p>%s</p>\n" % self._contents)

        elif name == "PERSONA":
            if self._elemstack[-1] == "PERSONAE":
                self._write("<li>%s</li>\n" % self._contents)
            else: # group
                self._write(self._contents + ", ")

        elif name == "GRPDESCR":
            self._write(self._contents + "</li>\n")
                
        elif name == "PERSONAE":
            self._write("</ul>\n")

        self._contents = ""
            
    def endDocument(self):
        self._write(bottom)

# --- Conversion function

def convert(infile, out):
    outf = open(out, "w")
    parse(infile, PlayToHtmlHandler(outf))
    outf.close()
        
# --- Main program

if __name__ == "__main__":
    convert(sys.argv[1], "out.html")
