
# --- ContentHandler

class RSS2HTMLHandler(handler.ContentHandler):

    def __init__(self, out, err):
        self._out = out
        self._err = err

        # tracking state
        self._data = ""
        self._first_item = 1
        self._first_elem = 1

        self._title = None
        self._link = None
        self._descr = None


    def startElement(self, name, attrs):
        if self._first_elem:
            if name != "rss":
                exc = _exceptions.SAXException("Document element must be "
                                               "'rss', not '%s'" % name)
                self._err.error(exc)
                
            self._first_elem = 0

            if attrs.get("version") != "0.91":
                exc = _exceptions.SAXException("Unknown RSS version: %s" %
                                               attrs.get("version"))
                self._err.warning(exc)


        self._data = ""

        if name == "item":
            if self._first_item:
                self._out.write(top %
                                (self._title, self._link, self._title))

                if self._descr != None:
                    self._out.write("<p>%s</p>" % self._descr)

                    self._out.write("\n<ul>\n")
                
            self._descr = None # reset for this item
            self._first_item = 0


    def endElement(self, name):
        if name == "title":
            self._title = self._data

        elif name == "link":
            self._link = self._data

        elif name == "description":
            self._descr = self._data

        elif name == "item":
            self._out.write('  <li><a href="%s">%s</a> %s\n' %
                            (self._link, self._title, self._descr or ""))

    def endDocument(self):
        self._out.write("</ul>\n")
        self._out.write(bottom)
            
    def characters(self, chars):
        self._data = self._data + chars


# --- Error handler

class RSSErrorHandler(handler.ErrorHandler):

    def __init__(self, outf):
        self._out = outf

    def fatalError(self, exception):
        self._out.write("<p class=error>Input was not well-formed: "
                        "%s</p>\n" % exception)
        raise exception

    def error(self, exception):
        self._out.write("<p class=error>Input was not valid: "
                        "%s</p>\n" % exception)
        raise exception


# --- Conversion function

def htmlconv_to_stream(infile, stream):
    err = RSSErrorHandler(stream)
    p = make_parser("xml.sax.drivers2.drv_xmlproc")
    p.setContentHandler(RSS2HTMLHandler(stream, err))
    p.setErrorHandler(err)
    p.setFeature(handler.feature_validation, 1)

    try:
        p.parse(infile)
        return 1
    except _exceptions.SAXException, e:
        sys.stderr.write("ERROR: Conversion aborted\n")
        sys.stderr.write(str(e) + "\n")
        return 0


import cStringIO
        
def htmlconv(infile, out):
    io = cStringIO.StringIO()
    if htmlconv_to_stream(infile, io):
        outf = open(out, "w")
        outf.write(io.getvalue())
        outf.close()

if __name__ == "__main__":
    htmlconv(sys.argv[1], sys.argv[2])
