# This script converts XBEL bookmark collections to web pages, using
# xbellib.

import xbellib, codecs

# --- Templates

top = \
"""
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">
<HTML>
<HEAD>
  <TITLE>%s</TITLE>
</HEAD>

<BODY>
<H1>%s</H1>
"""

bottom = \
"""        
<HR>

<ADDRESS>
Converted by xbel2html.py, using xbellib.
</ADDRESS>

</BODY>
</HTML>
"""

# --- HTML generator

def convert(infile, outfile):
    xbel = xbellib.load_xbel(infile)
    outf = codecs.open(outfile, "w", "iso-8859-1")
    title = xbel.get_title() or "Bookmark collection"
    outf.write(top % (title, title))
    if xbel.get_descr():
        outf.write("<p>%s</p>\n" % xbel.get_descr())

    convert_nodes(xbel.get_nodes(), outf, 1)
        
    outf.write(bottom)
    outf.close()

def convert_nodes(nodes, outf, level):
    bookmarks = filter(lambda node: isinstance(node, xbellib.Bookmark),
                       nodes)
    folders   = filter(lambda node: isinstance(node, xbellib.Folder),
                       nodes)

    if bookmarks and level < 4:
        outf.write("<ul>\n")
        for bookmark in bookmarks:
            outf.write('  <li><a href="%s">%s</a> %s\n' %
                       (bookmark.get_url(),
                        bookmark.get_title() or bookmark.get_url(),
                        bookmark.get_descr() or ""))
        outf.write("</ul>\n")

    if folders and level < 4:
        for folder in folders:
            outf.write("<h%d>%s</h%d>" %
                       (level + 1, folder.get_title(), level + 1))
            if folder.get_descr():
                outf.write("<p>%s</p>\n" % folder.get_descr())
            convert_nodes(folder.get_nodes(), outf, level + 1)

    if level > 3 and nodes:
        outf.write("<ul>\n")
        for node in bookmarks + folders:
            if isinstance(node, xbellib.Bookmark):
                outf.write('  <li><a href="%s">%s</a> %s\n' %
                           (node.get_url(),
                            node.get_title() or node.get_url(),
                            node.get_descr() or ""))
            elif isinstance(node, xbellib.Folder):
                outf.write('  <li><b>%s</b> %s\n' %
                           (node.get_title(), node.get_descr() or ""))
                convert_nodes(node.get_nodes(), outf, level + 1)
                
        outf.write("</ul>\n")
        
    
# --- Main program

import sys
convert(sys.argv[1], sys.argv[2])
