# A function for writing out XBEL objects as documents.

from xbellib_model import *
        
def serialize(xbel, out):
    out.write("<xbel>\n")
    _serialize_title(xbel, out)
    _serialize_nodes(xbel.get_nodes(), out, 1)
    out.write("</xbel>\n")

def _serialize_title(node, out, indent = "  "):
    if node.get_title():
        out.write("%s<title>%s</title>\n" % (indent, node.get_title()))
    if node.get_descr():
        out.write("%s<descr>%s</descr>\n" % (indent, node.get_descr()))
    
def _serialize_nodes(nodes, out, level):
    indent = "  " * level

    for node in nodes:
        if isinstance(node, Folder):
            if node.get_id():
                id = ' id="%s"' % node.get_id()
            out.write('%s<folder folded="%s"%s>\n' %
                      (indent,
                       {1 : "yes", 0 : "no"}[node.is_folded()], id))
            _serialize_title(node, out, "  " * (level + 1))
            out.write('\n')
            _serialize_nodes(node.get_nodes(), out, level + 1)
            out.write('%s</folder>\n\n' % indent)

        elif isinstance(node, Bookmark):
            out.write('%s<bookmark href="%s"' %
                      (indent, node.get_url()))
            if node.get_id():
                out.write(' id="%s"' % node.get_id())
            if node.get_added():
                out.write(' added="%s"' % node.get_added())
            if node.get_visited():
                out.write(' visited="%s"' % node.get_visited())
            out.write(">\n")
            
            _serialize_title(node, out, "  " * (level + 1))
            out.write('%s</bookmark>\n' % indent)            

        elif isinstance(node, Separator):
            out.write("%s<separator/>\n" % indent)

        elif isinstance(node, Alias):
            out.write('%s<alias ref="%s"/>\n' %
                      (indent, node.get_ref_node().get_id()))
