"""
An XML-generating module with an easy-to-use API.
"""

import cStringIO
from xml.sax.xmlreader import AttributesImpl
from xml.sax.saxutils import XMLGenerator

# --- The XML writer class

class XMLWriter:

    def __init__(self, cont_handler):
        self._cont_handler = cont_handler
        self._elemstack = []

    def startDocument(self):
        self._cont_handler.startDocument()
        
    def endDocument(self):
        self._cont_handler.endDocument()

    def startElement(self, name, attrs = {}):
        self._elemstack.append(name)
        self._cont_handler.startElement(name, AttributesImpl(attrs))

    def characters(self, chars):
        self._cont_handler.characters(chars)
    
    def endElement(self, name = None):
        self._cont_handler.endElement(name or self._elemstack[-1])
        del self._elemstack[-1]
        
    def processingInstruction(self, target, data):
        self._cont_handler.processingInstruction(target, data)
        
    def emptyElement(self, name, attrs = {}):
        self.startElement(name, attrs)
        self.endElement()

    def fieldElement(self, name, content, attrs = {}):
        self.startElement(name, attrs)
        self.characters(content)
        self.endElement()

    def fieldElementOptional(self, name, content, attrs = {}):
        if content:
            self.startElement(name, attrs)
            self.characters(content)
            self.endElement()
        
    # output generation control

    def close(self):
        if hasattr(self, "_fileobj"):
            self._fileobj.close()
    
    # retrievers

    def get_string_value(self):
        return self._string.getvalue()

    # contenthandler control

    def setContentHandler(self, handler):
        self._cont_handler = handler
    
# --- The namespace-aware XML writer class
    
class XMLWriterNS(XMLWriter):

    def __init__(self, cont_handler, default_ns):
        self._cont_handler = cont_handler
        self._default_ns = default_ns
        self._elemstack = []

    def startElement(self, name, attrs = {}):
        self._elemstack.append(name)
        # process attrs first!
        self._cont_handler.startElement((default_ns, name), name,
                                        AttributesImpl(attrs))
    
    def endElement(self, name = None):
        name = name or self._elemstack[-1]
        self._cont_handler.endElement((default_ns, name), name)
        del self._elemstack[-1]
    
# --- Convenience creation functions   

def file_writer(filename):
    return fileobj_writer(open(filename, "w"))

def string_generator():
    out = cStringIO.StringIO()
    xmlgen = XMLWriter(XMLGenerator(out))
    xmlgen._string = out
    return xmlgen

def fileobj_writer(fileobj):
    xmlgen = XMLWriter(XMLGenerator(fileobj))
    xmlgen._fileobj = fileobj
    return xmlgen

def sax_writer(cont_handler, lex_handler = None):
    return XMLWriter(cont_handler)
