<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                version="1.0">

  <xsl:template match = "rfc-index">
    <xsl:param name = "prefix" 
               select = "string(sites / site / prefix)" />
    <xsl:param name = "suffix" 
               select = "string(sites / site / suffix)" />


  <html>
    <head><title>An index of RFCs</title></head>

    <body>
    <h1>An index of RFCs</h1>

    <p><a href="#by-number">by number</a> |
       <a href="#by-standard">by standard</a> |
       <a href="#by-keyword">by keyword</a> |
       <a href="#by-author">by author</a> |
       <a href="#by-category">by category</a>
    </p>


    <h2><a name="by-number">RFCs by number</a></h2>
    <xsl:for-each select = "category / rfc">
      <xsl:sort select="number" data-type="number" order="ascending"/>

      <a href="#{number}"><xsl:value-of select="number"/></a> 
      <xsl:if test="not(position() = last())"> | </xsl:if>
    </xsl:for-each>


    <h2><a name="by-standard">RFCs by standard</a></h2>

    <table>
    <tr><th>Standard</th>  <th>RFCs</th></tr>
    <xsl:for-each select = "category / rfc / 
                      standard [not(preceding::standard = .)]">
      <xsl:sort select="." order="ascending"/>
      <xsl:variable name = "standard" select = "." />

      <tr>
      <td><xsl:value-of select = "." /></td>
      <td>
      <xsl:for-each select = "//rfc [standard = $standard]">
  	<xsl:sort select="number" data-type="number" order="ascending"/>
  
  	<a href="#{number}"><xsl:value-of select="number"/></a> 
  	<xsl:if test="not(position() = last())"> | </xsl:if>
      </xsl:for-each>
      </td></tr>
    </xsl:for-each>
    </table>


    <h2><a name="by-keyword">RFCs by keyword</a></h2>
    <table>
    <tr><th>Keyword</th>  <th>RFCs</th></tr>
    <xsl:for-each select = "category / rfc / keywords / 
                                 keyword [not(preceding::keyword = .)]">
      <xsl:sort select="." order="ascending"/>
      <xsl:variable name = "keyword" select = "." />

      <tr>
      <td><xsl:value-of select = "." /></td>
      <td>
      <xsl:for-each select = "//rfc [keywords / keyword = $keyword]">
  	<xsl:sort select="number" data-type="number" order="ascending"/>
  
  	<a href="#{number}"><xsl:value-of select="number"/></a> 
  	<xsl:if test="not(position() = last())"> | </xsl:if>
      </xsl:for-each>
      </td></tr>
    </xsl:for-each>
    </table>
  
    <h2><a name="by-author">RFCs by author</a></h2>
    <table>
    <tr><th>Author</th>  <th>RFCs</th></tr>
    <xsl:for-each select = "category / rfc / 
                                  author [not(preceding::author = .)]">
      <xsl:sort select="." order="ascending"/>
      <xsl:variable name = "author" select = "." />

      <tr>
      <td><xsl:value-of select = "." /></td>
      <td>
      <xsl:for-each select = "//rfc [author = $author]">
  	<xsl:sort select="number" data-type="number" order="ascending"/>
  
  	<a href="#{number}"><xsl:value-of select="number"/></a> 
  	<xsl:if test="not(position() = last())"> | </xsl:if>
      </xsl:for-each>
      </td></tr>
    </xsl:for-each>
    </table>

    <h2><a name="by-category">RFCs by category</a></h2>
    <xsl:for-each select = "category">
      <h3><xsl:value-of select="name"/></h3>

      <dl>
      <xsl:for-each select="rfc">
	<xsl:sort select="number" data-type="number" order="ascending"/>

        <dt><a name="{number}"><a href="{$prefix}{number}{$suffix}">RFC 
            <xsl:value-of select = "number"/></a></a></dt>
        <dd><xsl:call-template name="rfc-description"/>
            <xsl:if test="@obsoleted-by">
              <i>Obsoleted by 
              <a href="#{@obsoleted-by}"><xsl:value-of 
                                      select="@obsoleted-by"/></a>. </i>
            </xsl:if>
        </dd>
      </xsl:for-each>
      </dl>
    </xsl:for-each>
    </body>
  </html>
  </xsl:template>


  <xsl:template name = "rfc-description">
    <xsl:choose>
      <xsl:when test = "title">
        <xsl:value-of select = "title" />.
      </xsl:when>
      <xsl:when test = "nickname">
        <xsl:value-of select = "nickname" />.
      </xsl:when>
    </xsl:choose>

    <xsl:choose>
      <xsl:when test = "count(author) = 1">
  	By: <xsl:value-of select = "author"/>.
      </xsl:when>
      <xsl:when test = "author">
  	By: <xsl:apply-templates select = "author"/>
      </xsl:when>
    </xsl:choose>

    <xsl:apply-templates select = "date"/>
  </xsl:template>


  <xsl:template match = "number | nickname | keywords | standard"/>

  <xsl:template match = "date">
    <xsl:apply-templates/>.
  </xsl:template>

  <xsl:template match = "author">
    <xsl:apply-templates/>, 
  </xsl:template>

  <xsl:template match = "author [ position() = last() ]">
    and <xsl:apply-templates/>.
  </xsl:template>

</xsl:stylesheet>
