
import sys, string
from xml.sax.saxutils import XMLGenerator
from xml.xslt.Processor import Processor
import xbellib

# --- TextWriter to SAX

class SAXTextWriter:

    def __init__(self, content_handler):
        self._handler = content_handler
        self._name = None
        self._atts = {}
    
    def getResult(self):
        return None

    def startDocument(self):
        self._handler.startDocument()

    def endDocument(self):
        self._handler.endDocument()
    
    def text(self, text, escapeOutput=1):
        if self._name:
            self._startElement()
        self._handler.characters(text)

    def attribute(self, name, value, namespace=''):
        self._atts[name] = value

    def processingInstruction(self, target, data):
        if self._name:
            self._startElement()
        self._handler.processingInstruction(target, data)

    def comment(self, body):
        pass

    def startElement(self, name, namespace='', extraNss=None):
        if self._name:
            self._startElement()
            
        self._name = name
        self._atts = {}

    def endElement(self, name):
        if self._name:
            self._startElement()
        self._handler.endElement(name)

    def _startElement(self):
        self._handler.startElement(self._name, self._atts)
        self._name = None

# --- Opera bookmarks dumper

def write_opera_bookmarks(xbel, out):
    out.write("Opera Hotlist version 2.0\n\n")
    _serialize_nodes(xbel.get_nodes(), out)
    
def _serialize_nodes(nodes, out):
    for node in nodes:
        if isinstance(node, xbellib.Folder):
            out.write("#FOLDER\n")
            out.write("\tNAME=%s\n" %
                      string.join(string.split(node.get_title())))
            if node.get_descr():
                out.write("\tDESCRIPTION=%s\n" % node.get_descr())
            out.write('\n')
            _serialize_nodes(node.get_nodes(), out)
            out.write('-\n\n')

        elif isinstance(node, xbellib.Bookmark):
            out.write('#URL\n')
            out.write("\tNAME=%s\n" %
                      string.join(string.split(node.get_title())))
            out.write("\tURL=%s\n" % node.get_url())
            if node.get_descr():
                out.write("\tDESCRIPTION=%s\n" % node.get_descr())
            out.write("\n")

# --- Main program
            
builder = xbellib.XBELHandler()
proc = Processor()
proc.appendStylesheetUri(sys.argv[1])
writer = SAXTextWriter( builder )
proc.runUri(sys.argv[2], 0, {}, writer)

write_opera_bookmarks(builder.get_structure(), open("out.txt", "w"))
